﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using MigraDoc.DocumentObjectModel;
using MigraDoc.Rendering;
using MyFontResolver;
using PdfSharp.Fonts;
using PdfSharp.Pdf;

namespace MigraDocFontResolverDemo
{
    class Program
    {
        static void Main(string[] args)
        {
            // That's all it takes to register your own fontresolver
            GlobalFontSettings.FontResolver = new DemoFontResolver();

            // And now the slightly modified MigraDoc Hello World sample.

            // Create a MigraDoc document
            Document document = CreateDocument();
            document.UseCmykColor = true;

            // ===== Unicode encoding and font program embedding in MigraDoc is demonstrated here =====

            // A flag indicating whether to create a Unicode PDF or a WinAnsi PDF file.
            // This setting applies to all fonts used in the PDF document.
            // This setting has no effect on the RTF renderer.
            const bool unicode = false;

            // An enum indicating whether to embed fonts or not.
            // This setting applies to all font programs used in the document.
            // This setting has no effect on the RTF renderer.
            // (The term 'font program' is used by Adobe for a file containing a font. Technically a 'font file'
            // is a collection of small programs and each program renders the glyph of a character when executed.
            // Using a font in PDFsharp may lead to the embedding of one or more font programms, because each outline
            // (regular, bold, italic, bold+italic, ...) has its own fontprogram)
            const PdfFontEmbedding embedding = PdfFontEmbedding.Always;

            // ========================================================================================

            // Create a renderer for the MigraDoc document.
            PdfDocumentRenderer pdfRenderer = new PdfDocumentRenderer(unicode, embedding);

            // Associate the MigraDoc document with a renderer
            pdfRenderer.Document = document;

            // Layout and render document to PDF
            pdfRenderer.RenderDocument();

            // Save the document...
            const string filename = "HelloWorld.pdf";
            pdfRenderer.PdfDocument.Save(filename);
            // ...and start a viewer.
            Process.Start(filename);

        }

        /// <summary>
        /// Creates an absolutely minimalistic document.
        /// </summary>
        static Document CreateDocument()
        {
            // Create a new MigraDoc document
            Document document = new Document();

            SetupStyles(document);

            // Add a section to the document
            Section section = document.AddSection();

            // Add a paragraph to the section
            Paragraph paragraph = section.AddParagraph();

            paragraph.Format.Font.Color = Color.FromCmyk(100, 30, 20, 50);

            // Add some text to the paragraph
            paragraph.AddFormattedText("Hello, World!", TextFormat.Bold);

            section.AddParagraph("Hello, World!");

            // Demonstration for Heading styles.
            paragraph = section.AddParagraph("Hello, World! (Heading 1)");
            paragraph.Style = StyleNames.Heading1;

            paragraph = section.AddParagraph("Hello, World! (Heading 2)");
            paragraph.Style = StyleNames.Heading2;

            paragraph = section.AddParagraph("Hello, World! (Heading 3)");
            paragraph.Style = StyleNames.Heading3;

            paragraph = section.AddParagraph("Hello, World! (Heading 4)");
            paragraph.Style = StyleNames.Heading4;

            paragraph = section.AddParagraph();

            paragraph.Format.Font.Color = Color.FromCmyk(100, 30, 20, 50);

            // Add some text to the paragraph
            paragraph.AddFormattedText("Hello, World!", TextFormat.Bold);

            section.AddParagraph("Hello, World!"); 
            
            return document;
        }

        private static void SetupStyles(Document document)
        {
            // Default font for all styles.
            var style = document.Styles[StyleNames.Normal];
            style.Font.Name = "Ubuntu";

            // Overwrite font for headings 1 & 2.
            style = document.Styles[StyleNames.Heading1];
            style.Font.Name = "Janitor";
            style.Font.Size = 32;

            // Heading 2 inherits font from Heading 1.
            style = document.Styles[StyleNames.Heading2];
            style.Font.Size = 28;

            // Set normal font for Heading 3.
            style = document.Styles[StyleNames.Heading3];
            style.Font.Name = "Ubuntu";
            style.Font.Size = 24;

            style = document.Styles[StyleNames.Heading4];
            style.Font.Size = 20;
        }
    }
}
