﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using PdfSharp.Fonts;

namespace MyFontResolver
{
    /// <summary>
    /// Font resolver that extends the default implementation to handle the two font families we include in embedded resources.
    /// </summary>
    public class DemoFontResolver : FontResolverBase
    {
        public override FontResolverInfo ResolveTypeface(string familyName, bool isBold, bool isItalic)
        {
            // Ignore case of font names.
            var name = familyName.ToLower();

            // Deal with the fonts we know.
            switch (name)
            {
                case "ubuntu":
                    if (isBold)
                    {
                        if (isItalic)
                            return new FontResolverInfo("Ubuntu#bi");
                        return new FontResolverInfo("Ubuntu#b");
                    }
                    if (isItalic)
                        return new FontResolverInfo("Ubuntu#i");
                    return new FontResolverInfo("Ubuntu#");

                case "janitor":
                    return new FontResolverInfo("Janitor#");
            }

            // We pass all other font requests to the default handler.
            // When running on a web server without sufficient permission, you can return a default font at this stage.
            return base.ResolveTypeface(familyName, isBold, isItalic);
        }

        /// <summary>
        /// Return the font data for the fonts.
        /// </summary>
        public override byte[] GetFont(string faceName)
        {
            switch (faceName)
            {
                case "Janitor#":
                    return FontHelper.Janitor;

                case "Ubuntu#":
                    return FontHelper.Ubuntu;

                case "Ubuntu#b":
                    return FontHelper.UbuntuBold;

                case "Ubuntu#i":
                    return FontHelper.UbuntuItalic;

                case "Ubuntu#bi":
                    return FontHelper.UbuntuBoldItalic;
            }

            return base.GetFont(faceName);
        }
    }

    /// <summary>
    /// Helper class that reads font data from embedded resources.
    /// </summary>
    public static class FontHelper
    {
        public static byte[] Janitor
        {
            get { return LoadFontData("MyFontResolver.fonts.janitor.Janitor.ttf"); }
        }

        // Tip: I used JetBrains dotPeek to find the names of the resources (just look how dots in folder names are encoded).
        // Make sure the fonts have compile type "Embedded Resource". Names are case-sensitive.
        public static byte[] Ubuntu
        {
            get { return LoadFontData("MyFontResolver.fonts.ubuntufontfamily0._80.Ubuntu-R.ttf"); }
        }

        public static byte[] UbuntuBold
        {
            get { return LoadFontData("MyFontResolver.fonts.ubuntufontfamily0._80.Ubuntu-B.ttf"); }
        }

        public static byte[] UbuntuItalic
        {
            get { return LoadFontData("MyFontResolver.fonts.ubuntufontfamily0._80.Ubuntu-RI.ttf"); }
        }

        public static byte[] UbuntuBoldItalic
        {
            get { return LoadFontData("MyFontResolver.fonts.ubuntufontfamily0._80.Ubuntu-BI.ttf"); }
        }

        /// <summary>
        /// Returns the specified font from an embedded resource.
        /// </summary>
        static byte[] LoadFontData(string name)
        {
            var assembly = Assembly.GetExecutingAssembly();

            using (Stream stream = assembly.GetManifestResourceStream(name))
            {
                if (stream == null)
                    throw new ArgumentException("No resource with name " + name);

                int count = (int)stream.Length;
                byte[] data = new byte[count];
                stream.Read(data, 0, count);
                return data;
            }
        }
    }
}
